@@ -6,8 +6,8 @@
 ### Snapshot map (spplot) of an sts-object or matrix of counts
 ###
 ### Copyright (C) 2013-2014 Sebastian Meyer
-### $Revision: 889 $
-### $Date: 2014-04-05 23:52:40 +0200 (Sat, 05 Apr 2014) $
+### $Revision: 1125 $
+### $Date: 2014-12-05 14:11:14 +0100 (Fri, 05 Dec 2014) $
 ################################################################################
 
 ## x: "sts" or (simulated) matrix of counts
@@ -26,50 +26,33 @@ stsplot_space <- function (x, tps = NULL, map = x@map,
                            main = NULL, labels = FALSE,
                            at = 10, col.regions = NULL,
                            colorkey = list(space="bottom", labels=list(at=at)),
-                           total.args = NULL, sp.layout = NULL, ...)
+                           total.args = NULL, 
+                           gpar.missing = list(col="darkgrey", lty=2, lwd=2),
+                           sp.layout = NULL,
+                           xlim = bbox(map)[1, ], ylim = bbox(map)[2, ], ...)
 {
     counts <- if (inherits(x, "sts")) observed(x) else x
-    map <- map[colnames(x),] # assure same order
-
+    if (length(map) == 0L) stop("no map")
+    
     ## compute data to plot
     ncases <- getCumCounts(counts, tps)
     total <- sum(ncases)
     if (!is.null(population)) { # plot prevalence
         stopifnot(is.vector(population, mode="numeric"),
-                  length(population) == ncol(counts))
-        if (!is.null(colnames(counts)) && !is.null(names(population)))
-            stopifnot(colnames(counts) == names(population))
+                  length(population) == length(ncases))
         ncases <- ncases / population
         total <- total / sum(population)
     }
 
     ## add ncases to map@data
-    if (inherits(map, "SpatialPolygonsDataFrame")) {
-        map$ncases <- ncases
-    } else {
-        map <- SpatialPolygonsDataFrame(map, as.data.frame(ncases),
-                                        match.ID=FALSE)
-    }
-
+    map <- as(map, "SpatialPolygonsDataFrame")
+    map$ncases <- NA_real_
+    map$ncases[match(colnames(x),row.names(map))] <- ncases
+    
     ## default main title
     if (is.null(main) && inherits(x, "sts"))
         main <- stsTimeRange2text(x, tps)
 
-    ## get region labels
-    getLabels <- function (labels) {
-        if (isTRUE(labels)) {
-            row.names(map)
-        } else if (length(labels) == 1L &&
-                   (is.numeric(labels) | is.character(labels))) {
-            map@data[[labels]]
-        } else labels
-    }
-    labels.args <- if (is.list(labels)) {
-        labels
-    } else if (!is.null(labels) && !identical(labels, FALSE)) {
-        list(labels = getLabels(labels))
-    } # else NULL
-    
     ## check/determine color break points 'at'
     at <- checkat(at, ncases)
     ## default color palette
@@ -85,12 +68,12 @@ stsplot_space <- function (x, tps = NULL, map = x@map,
         colorkey <- modifyList(eval(formals()$colorkey), colorkey)
 
     ## automatic additions to sp.layout (region labels and total)
-    if (is.list(labels.args)) {
-        labels.args <- modifyList(list(x=coordinates(map), labels=TRUE),
-                                  labels.args)
-        labels.args$labels <- getLabels(labels.args$labels)
-        layout.labels <- c("panel.text", labels.args)
-        ## "panel.text" works since package "sp" imports all of "lattice"
+    if (is.list(gpar.missing) && any(is.na(map$ncases))) {
+        layout.missing <- c(list("sp.polygons", obj=map[is.na(map$ncases),]),
+                            gpar.missing)
+        sp.layout <- c(sp.layout, list(layout.missing))
+    }
+    if (!is.null(layout.labels <- layout.labels(map, labels))) {
         sp.layout <- c(sp.layout, list(layout.labels))
     }
     if (is.list(total.args)) {
@@ -107,9 +90,9 @@ stsplot_space <- function (x, tps = NULL, map = x@map,
     }
 
     ## generate the spplot()
-    args <- list(quote(map), "ncases", main=main,
+    args <- list(quote(map[!is.na(map$ncases),]), "ncases", main=main,
                  col.regions=col.regions, at=at, colorkey=colorkey,
-                 sp.layout=sp.layout, quote(...))
+                 sp.layout=sp.layout, xlim=xlim, ylim=ylim, quote(...))
     do.call("spplot", args)
 }
 
@@ -157,12 +140,13 @@ getCountIntervals <- function (nInt, data, trafo=scales::sqrt_trans(), ...) {
     at
 }
 
-stsTime2text <- function (stsObj, tps=TRUE)
-    paste(year(stsObj)[tps], epochInYear(stsObj)[tps], sep="/")
+stsTime2text <- function (stsObj, tps=TRUE, fmt="%i/%i") {
+    sprintf(fmt, year(stsObj)[tps], epochInYear(stsObj)[tps])
+}
 
-stsTimeRange2text <- function (stsObj, tps=NULL)
+stsTimeRange2text <- function (stsObj, tps=NULL, fmt="%i/%i", sep=" - ")
 {
     tpsRange <- if (is.null(tps)) c(1, nrow(stsObj)) else range(tps)
-    tpsRangeYW <- stsTime2text(stsObj, tpsRange)
-    paste(unique(tpsRangeYW), collapse=" - ")
+    tpsRangeYW <- stsTime2text(stsObj, tps=tpsRange, fmt=fmt)
+    paste0(unique(tpsRangeYW), collapse=sep)
 }
